function [f_vector, f_value] = Cone2HalfSpace()
% Demo for Cone/Half space interaction force calculation based on the paper
% Yang, P. and Qian, X., "A General, Accurate Procedure for Calculating
% Molecular Interaction Force," Journal of Colloid Interface Science, Submitted.
%
% Authors : Pinghai Yang, Xiaoping Qian
% Created : May. 01, 2009
% Version : 1.0
% Matlab Version : 7.0 ( Windows )
%
% Copyright (c) 2009 CDM Lab, Illinois Institute of Technology , U.S.A.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Load the parametrical data for the plane and the cone surfaces
%The structure of the NURBS surface followed the structure given in the IGS
%toolbox and NURBS toolbox of matlab (available at www.mathworks.com)
load plane;
load cone;

%Define the number of Gaussian Legendre quadrature points
%In this example we just use 2x2 quadrature points
quad_n = 2;
%This function is used to generate Gaussian Legendre quadrature points and weights
% with a prescribed number quad_n
[nodes weights] = quadg('gausslegendre', quad_n);
nodes = nodes(1:quad_n);
weights = weights(1,1:quad_n)'*2;

%Find the start and end parameters in u, v directions of the input Surface One.
start_u_plane = plane.order(1);
end_u_plane = size(plane.knots{1}, 2) - plane.order(1);

start_v_plane = plane.order(2);
end_v_plane = size(plane.knots{2}, 2) - plane.order(2);

%Find the start and end parameters in u, v directions of the input Surface Two.
%start_u_*
start_u_cone = cone.order(1);
end_u_cone = size(cone.knots{1}, 2) - cone.order(1);

start_v_cone = cone.order(2);
end_v_cone = size(cone.knots{2}, 2) - cone.order(2);

%Initialize the force vector for the surface to surface interaction force
f_vector = [0; 0; 0];

%Use i and j to go through all the patches in the Plane surface
for i = start_u_plane:end_u_plane,
    for j= start_v_plane:end_v_plane,

        %Find the start and end parameters in u, v directions of the current surface patch of the plane
        %determined by [u(i) u(i+1)]x[v(j) v(j+1)]
        u0_plane = plane.knots{1}(i);
        u1_plane = plane.knots{1}(i+1);
        v0_plane = plane.knots{2}(j);
        v1_plane = plane.knots{2}(j+1);

        %Use s and t to go through all the patches in the Plane surface
        for s = start_u_cone:end_u_cone,
            for t= start_v_cone:end_v_cone,

                %Find the start and end parameters in u, v directions of the current surface patch  of the cone
                %determined by [u(s) u(s+1)]x[v(t) v(t+1)]
                u0_cone = cone.knots{1}(s);
                u1_cone = cone.knots{1}(s+1);
                v0_cone = cone.knots{2}(t);
                v1_cone = cone.knots{2}(t+1);

                %Initialize the force vector temp_f for the patch to patch interaction force
                temp_f = 0;
                for ii = 1:quad_n,
                    %Changeof interval for Gaussian quadrature
                    u_p = (nodes(ii)+1)*(u1_plane-u0_plane)/2 + u0_plane;
                    for jj = 1:quad_n,
                        %Changeof interval for Gaussian quadrature
                        v_p = (nodes(jj)+1)*(v1_plane-v0_plane)/2 + v0_plane;
                        %Calculate the two first order partial derivatives
                        %with a given parameter pair (u_p, v_p)
                        %( tangent vectors) w.r.t u and v for the Plane surface
                        [temp_plane, dp_plane] = surf_der(plane, {u_p, v_p});
                        %Calculate the Plane surface normal with a given parameter pair (u_p, v_p) based on
                        %the cross product of the two tangent vectors
                        normal_plane = vec3cross(dp_plane{1}, dp_plane{2});
                        %Calculate the point on the Plane surfacewith a
                        %given parameter pair (u_p, v_p)
                        pnt_plane = surf_pnt(plane,{u_p v_p});

                        temp_f_0 = 0;
                        for ss = 1:quad_n,
                            %Changeof interval for Gaussian quadrature
                            u_c = (nodes(ss)+1)*(u1_cone-u0_cone)/2 + u0_cone;
                            for tt = 1:quad_n,
                                %Changeof interval for Gaussian quadrature
                                v_c = (nodes(tt)+1)*(v1_cone-v0_cone)/2 + v0_cone;
                                %Calculate the two first order partial derivatives
                                %with a given parameter pair (u_c, v_c)
                                %( tangent vectors) w.r.t u and v for the Cone surface
                                [temp_cone, dp_cone] = surf_der(cone, {u_c, v_c});
                                %Calculate the Cone surface normal with a given parameter pair (u_c, v_c) based on
                                %the cross product of the two tangent vectors
                                normal_cone = vec3cross(dp_cone{1}, dp_cone{2});
                                %Calculate the point on the Cone surfacewith a
                                %given parameter pair (u_c, v_c)
                                pnt_cone = surf_pnt(cone,{u_c v_c});

                                %Calculate the integral core f for the
                                %interaction force calculation, please
                                %refer to Eq. [20]& [21]
                                temp_f_0 = temp_f_0 + normal_cone'*(pnt_cone-pnt_plane)/3/norm(pnt_cone-pnt_plane)^6*weights(ss)*weights(tt);

                            end
                        end

                         %Calculate the integral core f for the
                         %interaction force calculation, please
                         %refer to Eq.[20]& [21]
                        temp_f = temp_f + temp_f_0*normal_plane*weights(ii)*weights(jj);

                    end
                end

                %The result force vector needs to be further weighted due to the changeof interval for Gaussian quadrature
                f_vector = f_vector + temp_f*(u1_plane-u0_plane)*(v1_plane-v0_plane)*(u1_cone-u0_cone)*(v1_cone-v0_cone)/16;
            end
        end

    end
end

% Atomic density for copper
Rho = 8.49 * 10^28;
% London-Van der Waals constant
C = 4.5639e-078;
% Unit change from nano-meter to meter
nano2meter = 1e9;

%Output force vector
f_vector = f_vector*C*Rho^2*nano2meter;
%Output force norm value
f_value = norm(f_vector);